/* 
 * adb.h
 * 
 * Komunikace s AD prevodniky pripojenymi na UART 
 *
 */

#ifndef _ADB_H_
#define _ADB_H_


#include "globals.h"

/* Ma se zakompilovat adb_rdid() ? */
//#define ADB_READID 

/* na kterem UARTu jsou ad desky pripojeny ? */
#ifndef __GNUC__
#define ADB_COM	0 
#else
#define ADB_COM	"/dev/ttyS1"
#endif
/* Timeout prijmu zprav (kdyz vyprsi, vypise se varovna hlaska na displej
 * a zkousi se to donekonecna znovu) */
#ifndef __GNUC__
#define ADB_TIMEOUT	(JIFRATE * 2)	/* [jiffies] */
#else
#define ADB_TIMEOUT	2		/* [seconds] */
#endif
/* Debughlasky ? Pise jaka chyba na kterem radku nastala. */
#define ADB_DEBUG

/**
 * Tohle nema prakticky zadne vyuziti, spis jen na otestovani, takze to neni
 * treba kompilovat pri realnem nasazeni 
 * @param addr adresa prevodniku
 * @param maxlen max. delka ident. vcetne koncoveho \0, musi byt > 0
 * @param result sem se ulozi vysledek (identifikator prevodniku)
 * @return kod chyby, 0 pro OK
 */
#ifdef ADB_READID
byte
adb_rdid(byte addr, byte maxlen, byte *result);
#endif

/** 
 * Precte kalibracni data a/d desky.
 * V pripade chybneho cteni muze ulozit do coef a off nesmysly!
 * @param addr	adresa desky
 * @param chn	kanal
 * @param coef	sem se ulozi koeficient
 * @param off	sem se ulozi offset
 * @return		0 -- OK, jinak chyba
 */
byte
adb_rdcalib(byte addr, byte chn, int *coef, int *off);

/**
 * Precte ocejchovane hodnoty
 *
 * Precte z desky ocejchovane hodnoty, vysledky ulozi do pole chnls, neuklada
 * vic nez *cnt hodnot. Pote do *cnt ulozi pocet skutecne prectenych kanalu. Do *status
 * ulozi stavovy bajt.
 *
 * @param	addr	adresa desky
 * @param	chnls	misto pro ulozeni vyslednych hodnot
 * @param	cnt		pri vstupu max. pocet kanalu, pri vystupu skutecny pocet odm. kanalu
 * @param	status	misto pro ulozeni stavoveho bajtu desky
 * @return	0: Ok, jinak chyba
 * TODO otestovat pro vic kanalu
 */
byte
adb_rdvals(byte addr, int *chnls, byte *cnt, byte *status);

/**
 * Precte prime neocejchovane 24-bitove hodnoty
 *
 * Precte z desky ocejchovane hodnoty, vysledky ulozi do pole chnls, neuklada
 * vic nez *cnt hodnot. Pote do *cnt ulozi pocet skutecne prectenych kanalu. Do *status
 * ulozi stavovy bajt.
 *
 * @param	addr	adresa desky
 * @param	chnls	misto pro ulozeni vyslednych hodnot
 * @param	cnt		pri vstupu max. pocet kanalu, pri vystupu skutecny pocet odm. kanalu
 * @param	status	misto pro ulozeni stavoveho bajtu desky
 * @return	0: Ok, jinak chyba
 * TODO otestovat pro vic kanalu
 */
byte
adb_rddirect(byte addr, dword *chnls, byte *cnt, byte *status);

/**
 * Zapise kalibracni hodnoty
 * @param addr adresa desky
 * @param chnl kalibrovany kanal 
 * @param koef kal. koeficient 
 * @param offs kal. offset
 * @return 0 OK jinak chyba
 */
byte
adb_wrcalib(byte addr, byte chnl, int koef, int offs);

/**
 * urcite neuskodi zavolat pred pouzivanim
 * @return 0 -- vse ok, jinak chyba
 */
byte
adb_init(void);

/**
 * ma smysl asi jen pro nekrabicovy aplikace (win, linux..), uvolni handly
 * apod..
 * @return 0 -- vse ok, jinak chyba
 */
byte
adb_done(void);

/**
 * Vraci kod posledni chyby komunikace s adb. dalsi volani vraci 0 (pokud
 * mezitim nenastala dalsi chyba, samozrejme).
 * Ma smysl hlavne proto, ze kazda chyba zpusobi zobrazeni errorove hlasky
 * na spodni radek, coz by aplikace urcite rada nekdy vycistila.
 * @return 0 -- vse ok, jinak chyba
 */
byte
adb_geterr(void);

#endif /* _ADB_H_ */
