/*
 * battery.h
 *
 * Monitorovani stavu baterek
 *
 * stav baterii je zjistovan a/d prevodem z pinu pripojeneho k Vcc.
 * battery_test() vraci bud ze baterky jsou ok, nebo jsou temer prazdne,
 * nebo se nevrati, hodi se do hlubokeho sleepu a vypise varovnou hlasku.
 * 
 */

#ifndef _BATTERY_H_
#define _BATTERY_H_

/* Je-li zadefinovano, battery_show() ukazuje take primou hodnotu z a/d 
 * prevodniku, vhodne pro ocejchovani baterii. Musi byt zadefinovano
 * i BATTERY_SHOW. */
//#define BATTERY_CALIBRATION

/* teoreticka max. kapacita akumulatoru */
#define BATTERY_MAX			390
/* namerim-li vic nez BATTERY_CHARGED, povazuji baterky za ok */
#define BATTERY_CHARGED		352 /* odpovidalo 11.5 V */
/* v intervalu mezi BATTERY_CHARGED a BATTERY_DISCHARGED jsou polovybite */
/* namerim-li mene nez BATTERY_DISCHARGED, povazuji baterie za vybite. */
#define	BATTERY_DISCHARGED	342 /* odpovidalo 11.0 V */
/* je-li zadefionavno, prikompiluje se i fce battery_show. */
//#define BATTERY_SHOW


/* kolikaty pin a jaky port je pripojen na napajeni ? */
#define BATPIN	7
#define BATPORT IO_ADER.bit.ADE7

/*
 * battery_geterr
 * Vraci, zda po poslednim battery_test() byly baterie vybite ci nikoliv.
 * Pri dalsich volanich battery_geterr() bez mezivolani battery_test vraci
 * vzdy false.
 * @return true: Baterie byly temer vybite, false: Baterie byly OK
 */
bool
battery_geterr(void);

/**
 * battery_test
 * Otestuje stav baterek A/D prevodem.
 * @param row -- radek na ktery se ma vypsat pripadne varovani
 * @return false -- baterky ok, true -- temer vybite
 */
bool
battery_test(byte row);

#ifdef BATTERY_SHOW
/**
 * ukaze na displeji bargraf a procentualni plnost baterii.
 * @param foo k nicemu
 * @return vzdy 0
 */
word
battery_show(void *foo);
#endif /* BATTERY_SHOW */

/**
 * @return hodnota a/d prevodniku na kterem je pripojeno napajeni 
 */
word
battery_status(void);

#endif /* _BATTERY_H_ */
