#include "globals.h"
#include "cmos.h"
#include "rtc.h"
#include "arm.h"
#include "panel.h"
#include "ui.h"

/* Changelog:
 * 2006-10-04: pokud se zadefinuje CMOS_TEST, prikompiluje se i fce cmos_test,
 * ktera nahodne cte a pise do CMOS a kontroluje zda si zapsane a prectene
 * bajty odpovidaji 
 */

/* adresy jsou v nibblech (v 15bajtove cmos jsou <0 .. 29>) */

/* Mam k dispozici 15 bajtu, kam chci ulozit nulu ramene (4b), checksum(1b), 4x
 * vzdalenost pro jednotlive programy (4x4b). To se samozrejme nevejde, ale
 * rameno ma rozsah max. asi +- 100 stupnu, a posledni rad milistupnu
 * (tisiciny) se stejne nepouziva, takze pokud nebudu ukladat tisiciny, staci
 * mi na ulozeni cisla zhruba od -10000 .. +10000. Pokud pouziji 16tibitova
 * cisla, mam rozsah -32768 .. 32767, coz mi s prehledem staci. Nulu radsi
 * ulozim jako 32bit, to se porad vejde.*/

static int16
_32to16(int32 x)
{
	return (int16)(x/10);
}

static int32
_16to32(int16 x)
{
	return ((int32)(x))*10;
}

bool
cmos_wr(void)
{
	register byte i;
	byte chksum;

	for (i = 0; i < 3; i++) {
		chksum = 0;
		if (
				cmos_wrdword(0 , arm_get_real_zero(), &chksum) ||
				cmos_wrword(8 , _32to16(arm_programs[0].dist), &chksum) ||
				cmos_wrword(12, _32to16(arm_programs[1].dist), &chksum) ||
				cmos_wrword(16, _32to16(arm_programs[2].dist), &chksum) ||
				cmos_wrword(20, _32to16(arm_programs[3].dist), &chksum) ||
				cmos_wrbyte(24, chksum, NULL)
		   )
			continue;
		return false;
	}
	return true;
}

bool
cmos_rd(void)
{
	register byte i;
	byte chksum, tmp8;
	int16 tmp16;
	int32 zero;
	
	for (i = 0; i < 3; i++) {
		chksum = 0;
		cmos_rddword(0 , (dword *)(&zero), &chksum);
		arm_set_real_zero(zero);
		cmos_rdword(8 , (word *)(&tmp16), &chksum);
		arm_programs[0].dist = _16to32(tmp16);
		cmos_rdword(12, (word *)(&tmp16), &chksum);
		arm_programs[1].dist = _16to32(tmp16);
		cmos_rdword(16, (word *)(&tmp16), &chksum);
		arm_programs[2].dist = _16to32(tmp16);
		cmos_rdword(20, (word *)(&tmp16), &chksum);
		arm_programs[3].dist = _16to32(tmp16);
		cmos_rdbyte(24, &tmp8, NULL);

		if (tmp8 == chksum)
			return false;
	}
	p_putscreen(
			"Chyba cteni z CMOS\n"
			"Nastavuji tovarni\n"
			"hodnoty. Stiskni <-", true);
	arm_set_zero(0);
	for (i = 0; i < sizeof(arm_programs)/sizeof(struct arm_program); i++)
		arm_programs[i].dist = 0;
	p_waitkey(K_ANY);
	return true;
}

#ifdef CMOS_TEST
/* Testovani CMOS, to se neztrati... */

void
puterr(const char *txt, byte addr, byte val)
{
	p_msg(txt);
	p_putidf(addr, 3);
	p_move(16, p_y);
	p_putidf(val, 3);
	jsleep(1000);
}

void
cmos_test(void)
{
	byte addr, val, retval;
	while(1) {
		addr = rand() % 29;
		val = rand() % 0xff;
		if (cmos_wrbyte(addr, val, NULL))
			puterr("W", addr, val);
		cmos_rdbyte(addr, &retval, NULL);
		if (retval != val)
			puterr("C", addr, val);
		p_msg("OK"); p_putidf(addr, 3);
	}
}
#endif 
