#include "menu.h"
#include "comfuncs.h"
#include "globals.h"
#include "sercom.h"
#include "block.h"
#include "panel.h"
#include "timer.h"
#include "datetime.h"
#include "misc.h"

uint32 sc_status;
char sc_fname[4];
void *sc_ptr;
static byte sc_tmpbuf[30]; /* Na prijem delsich bloku */

/* Ocekava prichod bloku s jednim uint32 -- statusem. */
enum sercom_err
sc_recv_status(uint32 *status)
{
	unsigned int len;
	enum sercom_err retval;
	byte buf[4];

	len = 4;
	if (err_ok != (retval = sercom_inblck(buf, &len)))
		return retval;
	if (len != 4)
		return err_unknown;
	*status = str_uint32(buf);
	return err_ok;
}

word
try_comfunc(struct comfunc *cf)
{
	enum sercom_err retval;

	p_msg(cf->text);
#ifdef TEMPO_CONNECTED
	do {
		retval = cf->func();
		if (retval != err_ok) {
			p_msg("Chyba:");
			p_puts(sercom_strerr(retval));
			jsleep(JIFRATE);
			p_msg(cf->text);
		}
	} while (retval != err_ok);
#endif 
	return 0;
}

word
try_comfunc_silent(enum sercom_err (*comfunc)(void))
{
	enum sercom_err retval;

#ifdef TEMPO_CONNECTED
	do {
		retval = comfunc();
		if (retval != err_ok) {
			p_msg("Chyba:");
			p_puts(sercom_strerr(retval));
			jsleep(JIFRATE);
		}
	} while (retval != err_ok);
#endif 
	return 0;
}

enum sercom_err
setipcfg(void)
{
	sercom_outf("siii", "SetIPCfg", ip, netmask, gateway);
	return sc_recv_status(&sc_status);
}
struct comfunc cf_setipcfg = {"Pisi nastaveni site..", setipcfg};

enum sercom_err
getipcfg(void)
{
	byte buf[12];
	int retval;
	unsigned int len = sizeof(buf);
	
	sercom_outstr("GetIPCfg");
	if (err_ok != (retval = sercom_inblck(buf, &len)))
		return retval;
	if (len != sizeof(buf))
		return err_unknown;
	sercom_scanf(buf, "iii", &ip, &netmask, &gateway);
	return err_ok;
}
struct comfunc cf_getipcfg = {"Ctu nastaveni site..", getipcfg};

enum sercom_err
getftpcfg(void)
{
	enum sercom_err retval;
	byte buf[60];
	unsigned int bufsize = sizeof(buf);
	char *server, *user, *pass;
	
	sercom_outstr("GetFTPCfg");
	if (err_ok != (retval = sercom_inblck(buf, &bufsize)))
		return retval;
	// blbe! memcpy(buf, "\tJmeno678\0\tUziv5678\0\tHeslo678\0", 30);
	sercom_scanf(buf, "sss", &server, &user, &pass);
	strncpyx(ftp_user, user, sizeof(ftp_user));
	strncpyx(ftp_pass, pass, sizeof(ftp_pass));
	strncpyx(ftp_server, server, sizeof(ftp_server));
	return err_ok;
}
struct comfunc cf_getftpcfg = {"Ctu nastaveni ftp..", getftpcfg};

enum sercom_err
setftpcfg(void)
{
	sercom_outf("ssss", "SetFTPCfg", ftp_server, ftp_user, ftp_pass);
	return sc_recv_status(&sc_status);
}
struct comfunc cf_setftpcfg = {"Pisi nastaveni ftp..", setftpcfg};

enum sercom_err
setdatetime(void)
{
	uint32 dt = pack_datetime(datocas);
	sercom_outf("si", "SetDateTime", dt);
	return sc_recv_status(&sc_status);
}
struct comfunc cf_setdatetime = {"Nastavuji hodiny....", setdatetime};

enum sercom_err
erasefiles(void)
{
	sercom_outstr("EraseFiles");
	return sc_recv_status(&sc_status);
}
struct comfunc cf_erasefiles = {"Mazu vsechny soubory", erasefiles};


enum sercom_err
gethello(char *buf, int maxlen)
{
	sercom_outstr("SayHello");
	return sercom_instr(buf, maxlen);
}


/* Otevre soubor fname pro zapis. Nastavuje sc_status. */
enum sercom_err
openfile(void)
{
	sercom_outf("ss", "Open", sc_fname);
	return sc_recv_status(&sc_status);
}
struct comfunc cf_openfile = {"Zakladam soubor...", openfile};

/* Uzavre soubor */
enum sercom_err
closefile(void)
{
	sercom_outstr("Close");
	return sc_recv_status(&sc_status);
}
struct comfunc cf_closefile = {"Zaviram soubor...", closefile};

/* Provede prikaz FindFirst, jmeno souboru ulozi do prom.
 * sc_fname. */
enum sercom_err
findfirst(void)
{
	sercom_outstr("FindFirst");
	return sercom_instr(sc_fname, sizeof(sc_fname));
}
struct comfunc cf_findfirst = {"Hledam prvni soubor", findfirst};

enum sercom_err
findnext(void)
{
	sercom_outstr("FindNext");
	return sercom_instr(sc_fname, sizeof(sc_fname));
}
struct comfunc cf_findnext = {"Hledam dalsi soubor", findnext};

enum sercom_err
findprev(void)
{
	sercom_outstr("FindPrev");
	return sercom_instr(sc_fname, sizeof(sc_fname));
}

enum sercom_err
transmit_delay(void)
{
	register uint32 delay = sc_status;
	enum sercom_err retval;
	sercom_outf("si", "TransmitDelay", delay);
	retval = sc_recv_status(&sc_status);
	if (retval != err_ok)
		sc_status = delay;
	return retval;
}

enum sercom_err
disk_free(void)
{
	sercom_outstr("DiskFree");
	return sc_recv_status(&sc_status);
}

enum sercom_err
describe_file(void)
{
	sercom_outstr("FileDescribe");
	return sercom_instr(sc_ptr, 21);
}

enum sercom_err
defaultftp(void)
{
	sercom_outstr("DefaultFTP");
	return sc_recv_status(&sc_status);
}

enum sercom_err
defaultip(void)
{
	sercom_outstr("DefaultIP");
	return sc_recv_status(&sc_status);
}

enum sercom_err
startftp(void)
{
	sercom_outstr("StartFTP");
	return sc_recv_status(&sc_status);
}

enum sercom_err
resetftp(void)
{
	sercom_outstr("ResetFTP");
	return sc_recv_status(&sc_status);
}

enum sercom_err
getftpinfo(void)
{
	enum sercom_err retval;
	unsigned int len = sizeof(sc_tmpbuf);

	sercom_outstr("GetFTPInfo");
	if (err_ok != (retval = sercom_inblck(sc_tmpbuf, &len)))
		return retval;
	sercom_scanf(sc_tmpbuf, "is", &sc_status, &sc_ptr);
	return retval;
}

enum sercom_err
startusb(void)
{
	sercom_outstr("StartUSB");
	return sc_recv_status(&sc_status);
}

enum sercom_err
resetusb(void)
{
	sercom_outstr("ResetUSB");
	return sc_recv_status(&sc_status);
}

enum sercom_err
getusbinfo(void)
{

	sercom_outstr("GetUSBInfo");
	return sc_recv_status(&sc_status);
}
