#ifndef _COMFUNCS_H_
#define _COMFUNCS_H_

#include "globals.h"

/*
 * terminologie:
 * fujitsu .. krabice od elsaca (klavesnice, displaj,
 * zde bude behat tento zdrojak)
 * pametova jednotka .. externi jednotka s kterou fujitsu
 * komunikuje po seriaku (pravdepodobne elsaco TEMPO)
 */

/* Tato struktura je parametrem funkce try_comfunc.
 * Text se vypise na obrazovku a vola se func().
 */
struct comfunc {
	const char *text;
	enum sercom_err (*func)(void);
};

/* Nektere funkce (napr. close, setipcfg,...) vraci jeden uint32, status.
 * Ten je po navratu ulozen zde, viz jednotlive fce. */
extern uint32 sc_status;

/* Dost funkci vyuziva tuto promennou pro predavani jmena souboru. */
extern char sc_fname[4];

/* Nektere funkce (napr. describe_file) pouzivaji pro ruzne ucely tento
 * pointer, viz konkretni dokumentace. */
extern void *sc_ptr;

/* Ocekava prichod bloku s jednim uint32 -- statusem. */
enum sercom_err
sc_recv_status(uint32 *status);

/* Vypise na displej cf->text a vola cf->func dokud tato nevrati 
 * err_ok. V pripade chyby vypise chybovou hlasku a jednu sekundu pocka 
 * pred opakovanim. Opakuje donekonecna, vraci vzdy 0. Kompatibilni
 * s user_func pro menusystem. 
 */
word
try_comfunc(struct comfunc *cf);

/* Stejne jako try_comfunc, ale nevypisuje zadny text na obrazovku
 * (pokud nedoslo k chybe, tu vypisuje) a jako parametr
 * bere rovnou funkci ktera se ma vykonat. */
word
try_comfunc_silent(enum sercom_err (*comfunc)(void));

/* Precte hello retezec pametove jednotky
 * Vraci err_ok, pokud vse probehlo v poradku, jinak nejakou
 * sercom_err podle chyby.
 */
enum sercom_err
gethello(char *buf, int maxlen);

/* Posle do pametove jednotky aktualni nastaveni site
 * (ip, netmask, gw)
 * Vraci err_ok, pokud vse probehlo v poradku, jinak nejakou
 * sercom_err podle chyby. Nastavuje sc_status.
 */
enum sercom_err
setipcfg(void);
extern struct comfunc cf_setipcfg; /* comfunc struktura pro setipcfg() */

/* Precte aktualni nastaveni site pametove jednotky
 * (ip, netmask, gw)
 * Vraci err_ok, pokud vse probehlo v poradku, jinak nejakou
 * sercom_err podle chyby.
 */
enum sercom_err
getipcfg(void);
extern struct comfunc cf_getipcfg; /* comfunc struktura pro getipcfg() */


/* Posle do pametove jednotky aktualni nastaveni ftp
 * Vraci err_ok, pokud vse probehlo v poradku, jinak nejakou
 * sercom_err podle chyby. Nastavuje sc_status.
 */
enum sercom_err
setftpcfg(void);
extern struct comfunc cf_setftpcfg; /* comfunc struktura pro setftpcfg() */

/* Precte aktualni nastaveni ftp z pametove jednotky
 * Vraci err_ok, pokud vse probehlo v poradku, jinak nejakou
 * sercom_err podle chyby.
 */
enum sercom_err
getftpcfg(void);
extern struct comfunc cf_getftpcfg; /* comfunc struktura pro getftpcfg() */

/* Nastavi datum a cas pametove jednotky podle aktualniho data
 * a casu fujitsu. Nastavuje sc_status.
 */
enum sercom_err
setdatetime(void);
extern struct comfunc cf_setdatetime; /* comfunc struct pro setdatetime()*/

/* Smaze vsechny soubory.
 * Nastavuje sc_status. */
enum sercom_err
erasefiles(void);
extern struct comfunc cf_erasefiles; /* comfunc struktura pro erasefiles */

/* Otevre soubor fname pro zapis. Nastavuje sc_status. */
enum sercom_err
openfile(void);
extern struct comfunc cf_openfile; /* comfunc struktura pro erasefiles */

/* Uzavre soubor */
enum sercom_err
closefile(void);
extern struct comfunc cf_closefile;

/* Provede prikaz FindFirst, jmeno souboru ulozi do prom.
 * sc_fname. */
enum sercom_err
findfirst(void);
extern struct comfunc cf_findfirst;

/* Provede prikaz FindNext, jmeno souboru ulozi do prom. sc_fname. */
enum sercom_err
findnext(void);
extern struct comfunc cf_findnext;

/* Provede prikaz FindNext, jmeno souboru ulozi do prom. sc_fname. */
enum sercom_err
findprev(void);

/* Provee prikaz TransmitDelay, jako parametr (milisekundy)
 * se vysle glob. promenna sc_status, navratovy status se
 * opet ulozi do sc_status.
 */
enum sercom_err
transmit_delay(void);

/* Po uspesnem vykonani prikazu je v sc_status ulozeno
 * volne misto pro ukladani dat na ext. jednotce v promilich
 * (0 .. 1000).
 */
enum sercom_err
disk_free(void);

/* Zavola fci FileDescribe na aktualni soubor. Vysledny popisujici
 * string (az 20 bajtu + '\0' na konci) je ulozen do pointru na ktery
 * ukazuje glob. promenna sc_ptr.
 */
enum sercom_err
describe_file(void);

/* Provede prikaz DefaultFTP */
enum sercom_err
defaultftp(void);

/* Provede prikaz DefaultIP */
enum sercom_err
defaultip(void);

/* Vysle prikaz StartFTP. Status se vraci v sc_status. */
enum sercom_err
startftp(void);

/* Vysle prikaz ResetFTP. Status se vraci v sc_status. */
enum sercom_err
resetftp(void);

/* Vysle prikaz GetFTPInfo. Status se vraci v sc_status, 
 * do sc_ptr se ulozi ukazatel na prijaty popisek (glob. stat. pole). */
enum sercom_err
getftpinfo(void);

/* Vysle prikaz StartUSB. Status se vraci v sc_status. */
enum sercom_err
startusb(void);

/* Vysle prikaz ResetUSB. Status se vraci v sc_status. */
enum sercom_err
resetusb(void);

/* Vysle prikaz GetUSBInfo. Status se vraci v sc_status. */
enum sercom_err
getusbinfo(void);

#endif /* _COMFUNCS_H_ */
