#include "globals.h"
#include "uart.h"
#include "gpib.h"
#include "panel.h"
#include "sercom.h"
#include "misc.h"
#include "comfuncs.h"
#include "block.h"
#include "timer.h"

#include "table.h" /* Kvuli hacku v gpib_read */

/*
 * Changelog: 
 * 2006-10-05: Pridano gpib_reset()
 * 2006-10-23: Pridano DEBUG
 * 2006-10-24: gpib_lacs(), tacs(), cacs(), ...
 * 2006-10-25: Zase predelano, udelano proste gpib_read, gpib_write a gpib_mim
 *             a hotovo :-)
 * 2006-10-30: DEBUG_LED/DEBUG_MSG
 * 2006-11-02: Nyni se dava UNL/UNT po skonceni kazde zpravy.
 * 2006-11-02: Zruseno gpib_wrdata, pridano gpib_get, gpib_dcl
 * 2006-11-16: Pridano gpib_is_nrfd_held
 */

/* Co se tyce funkce Controller: Podstatne si tu zjednodusuji praci.
 * Predpokladam, ze jsem jediny kontroler a ze jsem bud v (CACS), nebo v
 * (CSBS). pokud prechazim (CACS) -> (CSBS), muzu jit okamzite a jedine co
 * potrebuju udelat je shodit ATN, ktery jinak porad drzim v T. Pokud se chci
 * vratit zpatky, staci mi nahodit ATN a pockat T7 + T9 = 500ns + 1.5us = 2us a
 * jsem zpatky v CACS. Pouze pokud potrebuji zapsat nebo precist nejaka neATN
 * data, hodim se do (CSBS).
 *
 * Tento zdrojak uz je silne delany tak, ze se pocita s tim ze jsem Controller.
 * Veskery cas jsem pripraven na zapis, do rezimu cteni se hodim pouze ve
 * funkcich ktere skutecne neco ctou.
 *
 * Uplne na konci je hack, ktery rozslisuje zda komunikuji se stolem nebo s
 * luxmetrem a ma ruzne funkce pro zapis, protoze se mi nepodarilo udelat
 * rutinu ktera by dokazala bezchybne komunikovat s obojim.
 */

#define DONT_DEBUG_LED
#define DONT_DEBUG_MSG

#ifdef DEBUG_MSG


static void
put_dbg_char(char c)
{
	if (gpib_get_status() == GPIB_NONE)
		p_putc(c); 
	else 
		p_putc((c) + 'a' - 'A');
}
#define DBG_NL(x) p_msg(""); put_dbg_char(x)
#define DBG(x) put_dbg_char(x)

#else

#define DBG_NL(x)
#define DBG(x)

#endif /* DEBUG_MSG */

#ifdef DEBUG_LED

#define DBG_LED(num, col) p_setled(num, col)
/*
#define INDICATE_GPIB_ERR p_setled(4, LED_RED)
#define INDICATE_GPIB_OK  p_setled(4, LED_NONE)
*/

#define INDICATE_GPIB_ERR {p_setled(4, LED_RED); p_msg("THROW:"); p_putidf(__LINE__, 4); }
#define INDICATE_GPIB_OK  p_setled(4, LED_NONE)

#else

#define DBG_LED(num, col)
#define INDICATE_GPIB_ERR 
#define INDICATE_GPIB_OK 

#endif /* DEBUG_LED */

/* negativni logika */
#define T 0
#define F 1

/* draty v gpib_cntrl */
#define SRQ  ((gpib_cntrl & 0x01) == T)
#define ATN  ((gpib_cntrl & 0x02) == T)
#define EOI  ((gpib_cntrl & 0x04) == T)
#define DAV  ((gpib_cntrl & 0x08) == T)
#define NRFD ((gpib_cntrl & 0x10) == T)
#define NDAC ((gpib_cntrl & 0x20) == T)
#define IFC  ((gpib_cntrl & 0x40) == T)
#define REN  ((gpib_cntrl & 0x80) == T)

static uint16 gpib_exception;

#define GPIB_CLEAR_EXCEPTIONS {gpib_exception = GPIB_NONE; INDICATE_GPIB_OK;}
#define GPIB_IS_EXCEPTION_SET(e) (gpib_exception & (e))
#define TRY(cmd) {cmd; if (gpib_exception != GPIB_NONE) {goto catch_label; }}
#define THROW(e) {gpib_exception |= (e); INDICATE_GPIB_ERR; goto catch_label;}
#define CATCH catch_label

static dword gpib_timer;
#define TRY_USER_INTR \
	{ \
		if (jiffies > gpib_timer) { \
			THROW(GPIB_TIMEOUT);  \
		} \
	}


#define GPIB_ATN_T do {GPIB_ATN = T; usleep(2); } while(0)
#define GPIB_ATN_F (GPIB_ATN = F)

/* stav kontrolnich dratu po poslednim volani gpib_rdbyte() */
static byte gpib_cntrl; 

void
gpib_init(void)
{
	IO_ADER.byte = 0; /* Digi pristup */
		/* P10 do general purpose modu */
	IO_PPGC01.bit.PE00	= 0; /* PPG ch0 output pin enable */
	IO_PPGC01.bit.PE10	= 0; /* PPG ch1 output pin enable */
		/* P11 do general purpose modu */
	IO_PPGC23.bit.PE00	= 0; /* PPG ch0 output pin enable */
	IO_PPGC23.bit.PE10	= 0; /* PPG ch1 output pin enable */

	GPIB_DCNTRL = 0;
	GPIB_SETDATAIN;
	GPIB_DSRQ = 0;

	GPIB_IFC = F;
	GPIB_DIFC = 1;

	GPIB_DONRFD = 0;
	GPIB_DONDAC = 0;
	GPIB_ODAV = F;
	GPIB_DODAV  = 1;
	GPIB_ATN = F;
	GPIB_DATN = 1;
	GPIB_DTE = 1;
	GPIB_TE = 1;
	
	/* Remote enable jednou assertnu a drzim ho porad, pokud vubec chci mit
	 * kontrolu nad lokalnim ovladanim ostatnich pristroju tak mi ani nic
	 * jineho nezbyva, viz str. 37 + komentare (!) IEEE normy GPIB */
	GPIB_REN = T; 
	GPIB_DREN = 1;

	gpib_ifc();
}

/* Prepne sbernici do stavu pro zapis. Tady je taky dulezity spechat, nesmim
 * dat najevo DAV.
 */
static void
gpib_preparewr(void)
{
	DBG_LED(1, LED_RED);
	__DI();

	GPIB_DONRFD = 0;
	GPIB_DONDAC = 0;
	GPIB_ODAV = F;
	GPIB_ATN = F;
	/* Odsud */
	GPIB_DODAV = 1;
	GPIB_TE = 1; /* zapis */
	/* pocud by to melo trvat < 200ns */
	GPIB_DEOI = 1;
	GPIB_EOI = F;
	GPIB_SETDATAOUT;
	__EI();
}

void
gpib_reset(void)
{
	DBG('!');
	gpib_preparewr();
	GPIB_CLEAR_EXCEPTIONS;
}

uint16
gpib_get_status(void)
{
	return gpib_exception;
}

void
gpib_ifc(void)
{
	DBG('I');
	if (gpib_get_status() != GPIB_NONE) {
		gpib_reset();
	}
	GPIB_IFC = T;
	jsleep(1);
	GPIB_IFC = F;
}


/* precte jeden byte, vrati ho a stav kontrolnich dratu v okamziku cteni 
 * hodi do glob. promenne gpib_cntrl. Predpoklada
 * ze jsem v (CSBS), pripraven pro cteni.
 *
 * Mozne vyjimky:
 * GPIB_NONE
 * GPIB_TIMEOUT
 * GPIB_INTR .. preruseni uzivatelem
 */
static byte
gpib_rdbyte(void)
{
	register byte data;
	
	TRY();
	gpib_timer = jiffies + GPIB_MAXTIME;

	GPIB_ONDAC = T;
	GPIB_ONRFD = F;

	while(GPIB_IDAV != T)
		TRY_USER_INTR;
	GPIB_ONRFD = T;
	data = GPIB_GETDATA;
	gpib_cntrl = GPIB_CNTRL;
	GPIB_ONDAC = F;
	while(GPIB_IDAV == T)
		TRY_USER_INTR;
	GPIB_ONDAC = T;
CATCH:
	return data;
}

/* Zapise jeden byte. Predpoklada, ze bylo predtim provedeno
 * gpib_preparewr(). Pokud je eoi true, assertne pri vysilani EOI
 * drat. Predpoklada ze jsem v (CSBS), pripraven pro zapis.
 *
 * Mozne vyjimky:
 * GPIB_NONE
 * GPIB_TIMEOUT
 * GPIB_INTR
 *
 * Vraci:
 *  0 - poslal bajt ok.
 * -1 - prijemce nezije (NRFD == F && NDAC == F)
 * -2 - prijemce neni pripraven na data (NRFD == F)
 * -3 - prijemce neprijal data (NDAC == T)
 */
void
gpib_wrbyte(byte b, bool eoi)
{
	TRY();

	gpib_timer = jiffies + GPIB_MAXTIME;
	GPIB_EOI = eoi;

	gpib_exception &=~ GPIB_NOT_ALIVE;
	gpib_exception &=~ GPIB_NRFD;
	gpib_exception &=~ GPIB_NDAC;
	/* (SGNS) */
	GPIB_SETDATA(b);
	/* (SDYS) */
	/* Musim pockat T1 (2us), pak na RFD = T */
	usleep(2);
	/* Pokud nastane, je chyba na strane prijemce. */
	gpib_exception |= GPIB_NOT_ALIVE;
	while (GPIB_INRFD == F && GPIB_INDAC == F)
		TRY_USER_INTR;
	/* (STRS) */
	gpib_exception &=~ GPIB_NOT_ALIVE;
	gpib_exception |= GPIB_NRFD;
	while(GPIB_INRFD == T)
		TRY_USER_INTR;
	/* (SDYS) */
	gpib_exception &=~ GPIB_NRFD;
	gpib_exception |= GPIB_NDAC;
	GPIB_ODAV = T;
	while(GPIB_INDAC == T)
		TRY_USER_INTR;
	gpib_exception &=~ GPIB_NDAC;
CATCH:
	/* (SGNS) */
	GPIB_ODAV = F;
	GPIB_EOI = F;
}	

void
gpib_mim(byte msg)
{
	DBG('M');
	TRY();

	GPIB_ATN_T;
	gpib_wrbyte(msg, F);

CATCH:
	GPIB_ATN_F;
	return;
}

static void
gpib_cmd(char addr, byte cmd)
{
	GPIB_ATN_T;

	gpib_wrbyte(MIM_UNL, F);
	gpib_wrbyte(MIM_UNT, F);
	gpib_wrbyte(MIM_MLA(addr), F);
	gpib_wrbyte(cmd, F);

	GPIB_ATN_F;
}

void
gpib_gtl(char addr)
{
	DBG('G');
	gpib_cmd(addr, MIM_GTL);
}

void
gpib_get(char addr)
{
	DBG('T');
	gpib_cmd(addr, MIM_GET);
}

void
gpib_dcl(char addr)
{
	DBG('C');
	gpib_cmd(addr, MIM_DCL);
}

size_t
gpib_wrstr(byte addr, const char *str)
{
	const char *start = str;

	DBG('S');
	TRY();

	GPIB_ATN_T;
	gpib_wrbyte(MIM_UNL, F);
	gpib_wrbyte(MIM_UNT, F);
	gpib_wrbyte(MIM_MLA(addr), F);
	GPIB_ATN_F;
	TRY();

	while(*str) {
		TRY(gpib_wrbyte(*str, F));
		str++; /* Nemenit! musi vracet pocet zapsanych bajtu! */
	}
	gpib_wrbyte(0x0D, F);
	gpib_wrbyte(0x0A, T);

CATCH:
	return str - start;
}

/*
bool
gpib_is_nrfd_held(byte addr)
{
	const dword hold_test_time = JIFRATE/10;
	dword timer;

	DBG('N');
	TRY();

	GPIB_ATN_T;
	gpib_wrbyte(MIM_UNL, F);
	gpib_wrbyte(MIM_UNT, F);
	gpib_wrbyte(MIM_MLA(addr), F);
	GPIB_ATN_F;
	usleep(2);

	timer = jiffies + hold_test_time;
	while(jiffies < timer) {
		if (GPIB_INRFD == F)
			return false;
	}
CATCH:
	{}
	return true;
}
*/


/* 
 * Cte data az do EOI, vraci pocet prectenych dat. maxlen je max. pocet dat 
 * ktera se mohou precist. Nedava na konec '\0'.
 *
 * Uprava: konci take pri CRLF
 *
 * Mozne vyjimky:
 * GPIB_NONE
 * GPIB_INTR
 * GPIB_TIMEOUT
 * GPIB_CMDOFLOW
 */
size_t
gpib_read(byte addr, char *buf, size_t maxlen)
{
	int i = 0;

	DBG('?');
	TRY();

	GPIB_ATN_T;
	gpib_wrbyte(MIM_UNT, F);
	gpib_wrbyte(MIM_UNL, F);
	gpib_wrbyte(MIM_MTA(addr), F);

	/* Tady je dulezity pote co spustim ATN do 200ns nastavit na vystup
	 * NRFD a NDAC (viz funkce LE, prechazim do LACS). */
	__DI();
	GPIB_SETDATAIN;
	GPIB_DEOI = 0;
	GPIB_ONRFD = T; 
	GPIB_ONDAC = T;
	/* Odsud */
	GPIB_TE = 0; /* cteni */
	GPIB_DONRFD = 1;
	GPIB_DONDAC = 1;
	GPIB_ATN_F;
	GPIB_DODAV = 0; /* musi byt az po GPIB_TE = 0 */
	__EI();

	DBG_LED(1, LED_GREEN);

	do {
		if (i == maxlen) {
			THROW(GPIB_CMDOFLOW);
		}
		TRY(buf[i++] = gpib_rdbyte());
		if (i > 1 && buf[i-2] == 0x0d && buf[i-1] == 0x0a)
			goto finish;
	} while(!EOI);

CATCH:
finish:
	gpib_preparewr();
	/* Kdyz sem nedam tento UNTALK, tak stul hazi 01 Error pri dotazovacich
	 * funkcich (*IDN? apod...), ale luxmetr 100% funguje. Kdyz ho sem dam,
	 * stul se chova na 100% ok, ale luxmetr se driv nebo pozdeji zasekne
	 * na lux_set_range. 
	 *
	 * Takze to nakonec zkusim vyresit naprosto hnusnym zpusobem - divam
	 * se, jestli je zprava ctena z luxmetru nebo ze stolu a podle toho bud
	 * hodim nebo nehodim MIM_UNT. Je to uberhumus, ale uz ani trochu
	 * nechci stravit dalsi mesic zkoumanim toho kde jsou chyby v GPIB.*/
	if (addr == TABLE_ADDR) {
		GPIB_ATN_T;
		gpib_wrbyte(MIM_UNT, 0);
		GPIB_ATN_F;
	}
	return i;
}
