#ifndef _NUM2STR_H_
#define _NUM2STR_H_

/* Changelog:
 * 2006-09-19: Panelove funkce nyni definovany pouze pod CKDM.
 * 2006-09-19: s_ Makra nyni nejsou {}, ale pouze volani funkce.
 * 2006-10-17: Extraktnuto num2char
 */


#include "globals.h"

/*
**
**  On call:
**  num       =   number to convert
**  string    =   buffer for output
**  max_chars =   maximum size of buffer
**  base      =   number base for conversion.
**
**  Return value:
**  if illegal base
**    NULL
**  beginning of converted number.
**
**  notes: if number is too large in magnitude to fit in the buffer,
**  the MOST significant digits will be truncated.  If the number is
**  negative, a leading '-' will be placed in the buffer even if this
**  causes the most significant digit to be truncated.
**  The number is right justified in the buffer and the location of the
**  first character in the number is returned so:
**    If you want right justification, use the original string.
**    If you want left justification, use the returned string.
**  If the number doesn't fill the buffer:
**       leading characters will be filled with spaces.
**
**  public domain by Jerry Coffin

Modifikace: 
Pokud JE nejvyssi bit base nastaven na 0, nepise na konec \0
(a cislo v retezci je tedy posunuto o jeden znak doprava)
Pokud je druhy nejvyssi bit base nastaven na 1, doplni volne
misto pred cislem znakem '0' misto ' '.

*/

char *ltostr(long num, char *string, byte max_chars, byte base);
	
/* 0 -> '0', 9 -> '9', 12 -> 'C', ... */
char
num2char(byte num);

/* Par maker pro snazsi pouzivani:
 * [target]_puti[base][fill0][zero][flush]([dest], num, len)
 *
 * target je bud p (panel) nebo s (string)
 * base je bud d (decimal), h(hexadecimal) nebo b(binary)
 * fill0 je bud 0 nebo nic, pokud je 0, je volne misto pred vyslednym
 * 	cislem zaplneno '0', jinak mezerami.
 * zero je bud z nebo nic, pokud je to z, je na posledni misto
 * 	retezce umisten '\0' (tedy je to skutecby retezec :-).
 * flush ma smysl pouze pro panelove fce a je bud f anebo nic, pokud je f tak
 * 	se prislusna radka jeste p_flushrowne.
 * dest je bud ukazatel na string, nebo [x,y] souradnice na panelu, nebo
 * 	nic (akt. souradnice panelu)
 * num je cislo :-)
 * len je delka bufferu, vcetne mista na pripadne '\0'.
 *
 * priklad:
 * s_puti_(buf, 100, 5); v buf bude ' ','1','0','0','\0'
 * 
 * Zdaleka ne vsechny kombinace jsou opravdu zamakrovany, ale
 * neni problem si prislusnou kombinaci domakrovat, v t->oo
 * pravdepodobne budou zamakrovany skutecne vsechny :-)
 *** */
#define s_putid(str,num,len) ltostr((long)(num), str, len, 10)

#define s_putidz(str,num,len) ltostr((long)(num), str, len, 0x80 | 10)
	
#define s_putid0(str,num,len) ltostr((long)(num), str, len, 0x40 | 10)

#define s_putid0z(str,num,len) ltostr((long)(num), str, len, 0xC0 | 10)
	
#define s_putih(str,num,len) ltostr((long)(num), str, len, 16)

#define s_putihz(str,num,len) ltostr((long)(num), str, len, 0x80 | 16)
	
#define s_putih0(str,num,len) ltostr((long)(num), str, len, 0x40 | 16)

#define s_putih0z(str,num,len) ltostr((long)(num), str, len, 0xC0 | 16)

#ifdef __CKDM__

#include "panel.h"
	
#define p_putihf(num,len) {\
	ltostr((long)(num), p_vram[p_y]+p_x, len, 16); p_flushrow(p_y);}

#define p_putih0f(num,len) {\
	ltostr((long)(num), p_vram[p_y]+p_x, len, 0x40 | 16); p_flushrow(p_y);}

#define p_putib0f(num,len) {\
	ltostr((long)(num), p_vram[p_y]+p_x, len, 0x40 | 2); p_flushrow(p_y);}

#define p_putidf(num,len) {\
	ltostr((long)(num), p_vram[p_y]+p_x, len, 10); p_flushrow(p_y);}

#define p_putid0f(num,len) {\
	ltostr((long)(num), p_vram[p_y]+p_x, len, 0x40 | 10); p_flushrow(p_y);}

#define p_putihf(num,len) {\
	ltostr((long)(num), p_vram[p_y]+p_x, len, 16); p_flushrow(p_y);}

#define p_putid(num,len) ltostr((long)(num), p_vram[p_y]+p_x, len, 10)

#define p_putid0(num,len) ltostr((long)(num), p_vram[p_y]+p_x, len, 0x40 | 10)
	
#endif /* __CKDM__ */

#endif /* _NUM2STR_H_ */
