#include "p_num_x_x.h"
#include "sleep.h"
#include "panel.h"
#include "buzzer.h"

#define FRAC		((spec >> 0) & 0x0f)
#define INT			((spec >> 4) & 0x0f)
#define NEG			((spec & X_X_SIGNED) ? 1 : 0)
#define ZEROS_AFTER ((spec & X_X_ZEROS_AFTER) ? 1 : 0)

char *
p_getnum_x_x(char *buf, word spec) 
{
	byte key;
	byte start_pos = p_x;

	p_setcursor(C_BLOCK);
	for(;;) {
		go_sleep();
		key = p_getkey();
		if (key >= K_0 && key <= K_9) {
			if (p_x == start_pos && NEG) {
				buzzer_error();
				continue;
			}
			p_mvputc(p_x, p_y, key - K_0 + '0');
			if (p_x - start_pos > INT + FRAC + NEG)
				p_move(--p_x, p_y);
			if (p_x - start_pos == INT + NEG)
				p_move(++p_x, p_y);
		}
		if (key == K_UP || key == K_DOWN || key == K_ENTER ||
				(key >= K_F1 && key <= K_F5)) {
			p_ungetkey(key);
			return buf;
		}
		switch (key) {
			case K_ESC:
				p_ungetkey(key);
				return buf;
			case K_LEFT:
				if (p_x > start_pos)
					p_move(--p_x, p_y);
				if (p_x - start_pos == INT + NEG)
					p_move(--p_x, p_y);
				break;
			case K_RIGHT:
				if (p_x - start_pos < INT + FRAC + NEG)
					p_move(++p_x, p_y);
				if (p_x - start_pos == INT + NEG)
					p_move(++p_x, p_y);
				break;
			case K_MINUS: case K_PLUS:
				if (p_x != start_pos || !NEG) {
					buzzer_error();
					break;
				}
				p_putc(key == K_PLUS ? '+' : '-');
				break;
			case K_NONE:
				break;
		}
	}
}

int32
str2num_x_x(const char *num, word spec)
{
	int32 result = 0;
	int i;

	for (i = NEG; i < NEG + INT + FRAC + 1; i++) {
		if (i == NEG + INT) /* desetinna tecka */
			continue;
		result *= 10;
		if (num[i] == ' ')
			continue;
		if (num[i] < '0' || num[i] > '9')
			return -1;
		result += num[i] - '0';
	}
	if (NEG && num[0] == '-')
		return -result;
	return result;
}

void
p_put_x_x(int32 num, word spec)
{
	char *dest = p_vram[p_y] + p_x;
	register int i;
	char flag, tmp;
	
	if (NEG)
		*dest++ = num < 0 ? '-' : '+';
	if (num < 0)
		num = -num;
	dest += INT + FRAC;
	flag = 0;
	for (i = FRAC; i; i--) {
		tmp = num % 10;
		if (tmp || flag || i == 1 || ZEROS_AFTER) {
			flag = 1;
			tmp += '0';
		} else
			tmp = ' ';
		*dest-- = tmp;
		num /= 10;
	}
	*dest-- = '.';
	for (i = INT; i; i--) {
		*dest-- = num || i == INT ? ((num % 10) + '0') : ' ';
		num /= 10;
	}
	p_flushrow(p_y);
}
		
#undef INT
#undef FRAC
#undef NEG
