#include "parse_float.h"
#include "globals.h"

#ifdef __CKDM__
#include "string.h"
#endif

byte
parse_float(const char *s, float *f)
{
	float frac = 1.0;
	byte sgn = 0;	/* bit 0 .. minus u cisla, bit 1 .. minus u exponentu */
	signed char e = 0;
	
	*f = 0;
	
	while(*s == ' ')
		s++;
	if (*s == '+')
		s++;
	else if (*s == '-') {
		sgn = 1;
		s++;
	}
	if (*s != '.' && !isdigit(*s))
		return 0;

	for(;;) {
		if (*s == '.') {
			s++;
			goto rat;
		}
		if (tolower(*s) == 'e') {
			s++;
			goto E;
		}
		if (isdigit(*s)) {
			(*f) *= 10.0;
			(*f) += (float)(*s - '0');
			s++;
			continue;
		}
		goto end;
	}
rat:
	for(;;) {
		if (tolower(*s) == 'e') {
			s++;
			goto E;
		}
		if (isdigit(*s)) {
			frac /= 10.0;
			(*f) += (float)(*s - '0')*frac;
			s++;
			continue;
		}
		goto end;
	}
E:
	if (*s == '+')
		s++;
	else if (*s == '-') {
		sgn |= 2;
		s++;
	}
	while(isdigit(*s)) {
		e *= 10;
		e += *s - '0';
		s++;
	}
	while(e--) {
		if (sgn & 2)
			(*f) /= 10.0;
		else
			(*f) *= 10.0;
	}
end:
	if (sgn & 1)
		(*f) = -(*f);
	
	*f = (*f);
	return 1;
}

#ifdef TEST_FLOAT

#include <stdlib.h>
#include <stdio.h>

#define ABS(x) (x) < 0 ? (-(x)) : (x)
void
test_convert(const char *s, float result)
{
	float parse_result;
	const float epsilon = 0.00000001;
	
	if (!parse_float(s, &parse_result)) {
		fprintf(stderr, "FAILED parsing %s\n", s);
		return;
	}
	if (abs(parse_result - result) > epsilon) {
		fprintf(stderr, "FAILED parsing %s, should be %f, was %f\n",
		        s, result, parse_result);
		return;
	}
	fprintf(stderr, "Passed %s (should be %f, was %f)\n", 
		        s, result, parse_result);
}

void
test_float(void)
{
	test_convert("1", 1.0);
	test_convert(" 1.", 1.0);
	test_convert("1.0", 1.0);
	test_convert(" 1.12345", 1.12345);
	test_convert(".12345", 0.12345);
	test_convert(" 0.12345", 0.12345);
	test_convert("-1", -1.0);
	test_convert(" -1.12345", -1.12345);
	test_convert("-.12345", -0.12345);
	test_convert(" -0", 0);
	test_convert("1.001", 1.001);
	test_convert(" -.000123", -0.000123);
	test_convert("1e3", 1000.0);
	test_convert(" .1e3", 100.0);
	test_convert("0.1e-3", 0.0001);
	test_convert(" -.012e-2", -0.00012);
	test_convert("-12.12e-2", -0.1212);
	test_convert("1000000000e-11", 0.01);
	test_convert("-0.00000001E10", -100);
	test_convert("+1.2E+1", 12);
}

int 
main(int argc, char *argv[])
{
	test_float();
}
#endif 
