/* ava
 *
 * Knihovna pro komunikaci pres seriak na linuxu. Neni hotova, je to jen
 * vytrhanej starej sercom.c !!!
 *
 *
 * Changelog:
 *
 * 2006-09-14: Zalozeno (vytrhani ze sercom.c)
 */

/* Linux **********************************************************************/

#ifdef __linux__

#include <stdio.h>
#include <unistd.h>
#include <termios.h>
#include <sys/types.h>
#include <sys/time.h>
#include <fcntl.h>
#include <errno.h>
#include <string.h>


int SERCOM_TIMEOUT;
static int comout, comin;

byte
sercom_init(enum sercom_inittype inittype)
{
	SERCOM_TIMEOUT = 2;
	switch(inittype) {
		case fifo_server:
			comin = open("fifo_ts", O_RDONLY  | O_NONBLOCK  , 0);
			if (comin == -1) {
				perror("open");
				return 1;
			}
			comout = open("fifo_st", O_WRONLY, 0);
			if (comout == -1) {
				perror("open");
				return 1;
			}
			break;
		case fifo_tester:
			comout = open("fifo_ts", O_WRONLY, 0);
			if (comout == -1) {
				perror("open");
				return 1;
			}
			comin = open("fifo_st", O_RDONLY  | O_NONBLOCK , 0);
			if (comin == -1) {
				perror("open");
				return 1;
			}
			break;
		case ttys0:
			comout = comin = open("/dev/ttyS0", O_RDWR | O_NONBLOCK | O_NOCTTY, 0);
			if (comin == -1) {
				perror("open");
				return 1;
			}
			break;
		case ttys1:
			comout = comin = open("/dev/ttyS1", O_RDWR | O_NONBLOCK | O_NOCTTY, 0);
			if (comin == -1) {
				perror("open");
				return 1;
			}
			break;
	}	
	return 0;
}

byte
sercom_done(void)
{
	int eq = comin == comout;

	if (close(comin) == -1) {
		perror("close");
		return 1;
	}
	if (!eq && close(comout) == -1) {
		perror("close");
		return 1;
	}
	return 0;
}

void
sercom_out(const void *data, size_t len)
{
	int ret;
	const byte *p = data;
#ifdef DEBUGCOM
    int i;
	fprintf(stderr,"\n>>>\n");
#endif
	
	while (len) {
		ret = write(comout, p, len);
		if (ret == -1) {
			perror("write");
			return;
		}
#ifdef DEBUGCOM
		fprintf(stderr,"Written:%i\n", ret);
        for (i = 0; i < ret; i++) 
            fprintf(stderr,"(%c %.2x)", ((char *)p)[i], ((char *)p)[i]);
#endif
		len -= ret;
		p += ret;
	}
}

enum sercom_err
sercom_in(void *data, size_t len)
{
	fd_set	set;
	struct timeval timeout;
	int ret;
	byte *p = data;
#ifdef DEBUGCOM
    int i;
    fprintf(stderr,"\n<<<\n");
#endif

	while(len) {
		FD_ZERO(&set);
		FD_SET(comin, &set);
		timeout.tv_sec = SERCOM_TIMEOUT;
		timeout.tv_usec = 0;
		switch(select(FD_SETSIZE, &set, NULL, NULL, &timeout)) {
			case 0: return err_timeout; /* timeout */
			case -1: 
				perror("select");
				return err_unknown; /* error */
			case 1: break; /* input ready */
		}
		ret = read(comin, p, len);
		if (ret == -1) {
			perror("read");
			return err_unknown;
		}
#ifdef DEBUGCOM
        for (i = 0; i < ret; i++) 
            fprintf(stderr,"(%c %.2x)", ((char *)p)[i], ((char *)p)[i]);
#endif
		len -= ret;
		p += ret;
	}
	return err_ok;
}

void
sercom_flushin(void)
{
	char buf;
	fd_set	set;
	struct timeval timeout;
	int ret;

#ifdef DEBUG
	fprintf(stderr, "sercom_flushin()!\n");
#endif 

	while(1) {
		FD_ZERO(&set);
		FD_SET(comin, &set);
		timeout.tv_sec = 0;
		timeout.tv_usec = 1;
		switch(ret = select(FD_SETSIZE, &set, NULL, NULL, &timeout)) {
			case 0: return;
			case -1: 
				perror("select");
				return; /* error */
			case 1: read(comin, &buf, 1); /* input ready */
		}
	}
	usleep(SERCOM_TIMEOUT * 1000000UL);
}

#endif /* __linux__ */

