/* 
 * Knihovna pro praci se seriovym portem, verze pro WIN32
 *
 * Changelog
 * 2006-09-14/ava: Zalozeno 
 */

/* debugovani komunikace (ser_in, ser_out) */

#define DEBUGUART

#include <assert.h>
#include <windows.h>
#include <stdio.h>

#include "globals.h"
#include "uart.h"

#ifdef DEBUG
#include <stdio.h>
#include <stdlib.h>
#endif

#define MAX_PORTS 2

/* Handly seriovych portu */
static HANDLE handles[MAX_PORTS]; 
/* Ukazatele do prostoru, kde mam zapisovat prectena data */
static char *rdbufs[MAX_PORTS];   
/* Kolik dat mam jeste precist. Pokud je 0, cteni neprobiha. */
static int rdleft[MAX_PORTS];
/* Kolik dat uz jsem precetl. */
static int rdcounts[MAX_PORTS];

static void
msgbox_error(const char *txt)
{
	MessageBox(NULL, TEXT(txt), TEXT("Chyba"), MB_OK|MB_ICONERROR);
}

static void 
print_err(const char *func) 
{ 
	TCHAR szBuf[80]; 
	LPVOID lpMsgBuf;
	DWORD dw = GetLastError(); 

	FormatMessage(
			FORMAT_MESSAGE_ALLOCATE_BUFFER | 
			FORMAT_MESSAGE_FROM_SYSTEM,
			NULL,
			dw,
			MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
			(LPTSTR) &lpMsgBuf,
			0, NULL );

	wsprintf(szBuf, "%s failed with error %d: %s", func, dw, lpMsgBuf);
	printf(szBuf);
	printf("\n");

	msgbox_error(szBuf);
	LocalFree(lpMsgBuf);
}

static int ser_timeout;

static int
set_timeout(int kanal, int timeout /* [ms] */)
{
	COMMTIMEOUTS timeouts;

		/* viz dokumentace SDK k COMMTIMEOUTS, Remarks */
	if (timeout == 0) {
		timeouts.ReadIntervalTimeout = MAXDWORD;
		timeouts.ReadTotalTimeoutMultiplier = MAXDWORD;
		timeouts.ReadTotalTimeoutConstant = 0;
	} else {
		timeouts.ReadIntervalTimeout = 0;
		timeouts.ReadTotalTimeoutMultiplier = 0;
		timeouts.ReadTotalTimeoutConstant = timeout;
	}
	
	timeouts.WriteTotalTimeoutMultiplier = 0;
	timeouts.WriteTotalTimeoutConstant = 0;
	if (!SetCommTimeouts(handles[kanal], &timeouts)) {
		print_err("SetCommTimeouts");
		return 1;
	}
	ser_timeout = timeout;
	return 0;
}

int
ser_init(byte kanal, long baud, char parita)
{
	DCB dcb;
	const char *portname;

	if (kanal == 0)
		portname = "COM1";
	else if (kanal == 1)
		portname = "COM2";
	else
		return 1;
	   
	handles[kanal] = CreateFile(portname,  GENERIC_READ | GENERIC_WRITE, 0, 0,
                         OPEN_EXISTING, FILE_FLAG_WRITE_THROUGH, 0);
	if (handles[kanal] == INVALID_HANDLE_VALUE) {
		print_err("CreateFile");
		return 1;
	}
			
	if (!GetCommState(handles[kanal], &dcb)) {
		print_err("GetCommState");
		return 1;
	}
	
	dcb.BaudRate = baud;
	dcb.fBinary = true;
	dcb.fParity = parita ? true : false;
	dcb.fOutxCtsFlow = false;
	dcb.fOutxDsrFlow = false;
	dcb.fDtrControl = false;
	dcb.fDsrSensitivity = false;
	//dcb.fTXContinueOnXoff; 1; /* Continue TX when Xoff sent */
	dcb.fOutX = false;
	dcb.fInX = false;
	dcb.fErrorChar = false;
	dcb.fNull = false;
	dcb.fRtsControl = false;
	dcb.fAbortOnError = false;
	dcb.ByteSize = 8;
	dcb.Parity = parita;
	dcb.StopBits = 0;
	
	if (!SetCommState(handles[kanal], &dcb)) {
		print_err("SetCommState");
		return 1;
	}

	return set_timeout(kanal, 0);
}

int
ser_out(byte kanal, const void *co, int kolik)
{
	unsigned long int wr, x;
	char *data = (char *)co;

#ifdef DEBUGUART
	printf("> ");
	for (x = 0; x < kolik; x++) {
		wr = data[x];
		if (wr < 32 || wr > 128)
			printf("[%lx] ", wr);
		else
			printf("%c(%lx) ", (int)wr, wr);
	}
	printf("\n");
#endif
	
	wr = 0;
	while(wr < kolik) {
		if (!WriteFile(handles[kanal], data+wr, kolik - wr, &x, 0)) {
			print_err("WriteFile");
			return 1;
		}
		wr += x;
	}
	return 0;
}

/* Zkusi precist (pokud je to treba) dalsi bajty z prislusneho kanalu */
static int
rdchunk(byte kanal)
{
	long unsigned int rd;

	if (!ReadFile(handles[kanal], rdbufs[kanal], rdleft[kanal], &rd, NULL)) {
		print_err("ReadFile");
		return 1;
	}
	rdbufs[kanal] += rd;
	rdleft[kanal] -= rd;
	rdcounts[kanal] += rd;

#ifdef DEBUGUART
	int i;

	printf("< ");
	for (i = 0; i < rd; i++) {
		unsigned char x = ((char *)(rdbufs[kanal]))[i - rd];
		if (x < 32 || x > 128)
			printf("[%x] ", x);
		else
			printf("%c(%x) ", x, x);
	}
	printf("\n");
#endif
		
	return 0;
}

int
ser_in(byte kanal, void *kam, int kolik)
{
	rdbufs[kanal] = kam;
	rdleft[kanal] = kolik;
	rdcounts[kanal] = 0;
	return rdchunk(kanal);
}

int
ser_in_stop(byte kanal)
{
	rdbufs[kanal] = NULL;
	rdleft[kanal] = 0;
	rdcounts[kanal] = 0;
	return 0;
}

int
ser_ilen(byte kanal)
{
	rdchunk(kanal);
	return rdcounts[kanal];
}

int 
ser_olen(byte kanal)
{
	return 0;
}

int
ser_cts(byte kanal)
{
	assert(!"ser_cts neni implementovano");
}

int 
ser_istat(byte kanal)
{
	rdchunk(kanal);
	return (rdleft[kanal] > 0) ? 1 : 0;
}

static int
ser_olen(byte kanal)
{
	unsigned long int errors;
	COMSTAT comstat;

	if (!ClearCommError(handles[kanal], &errors, &comstat)) {
		print_err("ClearCommError");
		return 0;
	}
	return comstat.cbOutQue;
}

int 
ser_ostat(byte kanal)
{
	return ser_olen(kanal) > 0;
}

void
ser_flushbuf(byte kanal)
{
	unsigned long int rd;
	char buf[10];
	int last_timeout = ser_timeout;
	
	set_timeout(kanal, 1);
	while(1) {
		if (!ReadFile(handles[kanal], buf, sizeof(buf), &rd, NULL)) {
			print_err("ReadFile");
			break;
		}
		if (!rd)
			break;
	}
	set_timeout(kanal, last_timeout);	
}

void
ser_purgebuf(byte kanal, uint16 time)
{
	unsigned long int rd;
	char buf[10];
	int last_timeout = ser_timeout;
	
	set_timeout(kanal, time*1000/JIFRATE);
	while(1) {
		if (!ReadFile(handles[kanal], buf, sizeof(buf), &rd, NULL)) {
			print_err("ReadFile");
			break;
		}
		if (!rd)
			break;
	}
	set_timeout(kanal, last_timeout);	
}

int
ser_ibuflen(byte kanal)
{
	unsigned long int errors;
	COMSTAT comstat;

	if (!ClearCommError(handles[kanal], &errors, &comstat)) {
		print_err("ClearCommError");
		return 0;
	}
	return comstat.cbInQue;
}

bool
ser_isoverflow(byte kanal)
{
	return false;
}

int
ser_in_timeout(byte kanal, void *data, size_t len, uint16 timeout)
{
	int last_timeout = ser_timeout;
	unsigned long rd, total_rd;
	unsigned char *dst = (char *)data;
	
#ifdef DEBUGUART
	int i;

	printf("< ");
#endif
	set_timeout(kanal, timeout*1000/JIFRATE);
	for (total_rd = 0; total_rd < len; ) {
		if (!ReadFile(handles[kanal], dst, len, &rd, NULL)) {
			print_err("ReadFile");
			total_rd += rd;
			break;
		}
		if (!rd)
			break;
#ifdef DEBUGUART
		for (i = 0; i < rd; i++) {
			unsigned char x = dst[i];
			if (x < 32 || x > 128)
				printf("[%x] ", x);
			else
				printf("%c(%x) ", x, x);
		}
#endif
		total_rd += rd;
		dst += rd;
		len -= rd;
	}
#ifdef DEBUGUART
	printf("\n");
#endif
	set_timeout(kanal, last_timeout);
	return total_rd;
}
