#ifndef _UART_H_
#define _UART_H_

/* Changelog
 *
 * 2006-09-14: Update dokumentace, kompatibilita s WIN32 verzi
 * 2006-10-17: Pridano UART_SIGNAL_ERRORS
 */

#include "globals.h"

/* Pokud je zadefinovano, pri vyskytu chyby rozsviti LED a vypise chybu na
 * spodni radek.  Doporucuji, zvysuje sanci ze pripadne chyby budou odhaleny uz
 * pri psani programu a ne az u zakaznika, krome toho jsem prave stravil 4
 * hodiny hledanim chyby ktera by se pri zadefinovani tohoto makra odhalila
 * sama. */
#define UART_SIGNAL_ERRORS

/* Pro ktere seriove porty chceme generovat kod ? */
#define UART0 
#define UART1

/* Velikost softwaroveho bufferu uart0 a uart1, pokud je 0 nemaji zadny 
 * buffer. Pokud maji nejaky buffer, jsou neustale povolena rx preruseni.
 */
#define UART0_BUF	20
#define UART1_BUF	40 /* Musi se mi sem vejit nejdelsi prikaz co prijde po seriaku. */

#ifdef __CKDM__

/* dva odchozi buffery */
extern volatile const char *txptr[2];
/* dvakrat pocty bajtu k odeslani */
extern volatile int txcnt[2];

/* dva prichozi buffery */
extern volatile char *rxptr[2];
/* dvakrat pocet prijatych bajtu a max. pocet bajtu k prijeti */
extern volatile int rxcnt[2], rxmax[2];
/* dvakrat zda jsme v prijimacim modu */
extern volatile char rxrun[2];

#endif /* __CKDM__ */

/* Parita: 0 - zadna, 1 - licha, 2 - suda, dale je nastaven 2 stopbit a 
 * 8-bitovy prenos */
/* inicializuje ser. kanal */
int 
ser_init(byte kanal,long baud,char parita);

/* zacne vysilat data na seriak. V prubehu vysilani (konec lze zjistit pomoci
 * ser_ostat) se nesmi prepisovat data v *co ani zacit vysilat jina data! */
int 
ser_out(byte kanal,const void *co,int kolik);

/* zacne vysilat data na seriak. Vraci se teprve pote, co jsou vsechna data
 * odeslana.  */
int 
ser_out_safe(byte kanal,const void *co,int kolik);

/* zacne prijimat data ze seriaku. Konec prijmu se MUSI zjistovat pomoci
 * ser_istat nebo ser_ilen, pod WIN32 tyto funkce volaji ReadLine a data by
 * jinak prijata nebyla. */
int 
ser_in(byte kanal,void *kam,int kolik);

/*
 * Precte len bajtu do data. 
 * Pokud pri cekani nastala prodleva mezi daty vetsi nez timeout jiffies,
 * vraci se.
 *
 * vraci pocet prijatych bajtu.
 */
int
ser_in_timeout(byte kanal, void *data, size_t len, uint16 timeout);

/* zastavi prijimani dat ze seriaku */
int 
ser_in_stop(byte kanal);

/* Kolik dat uz bylo prijmuto pro posledni volani ser_in() */
int 
ser_ilen(byte kanal);

/* Kolik dat uz bylo odeslano pro posledni volani ser_out() */
int 
ser_olen(byte kanal);

/* vraci zda je CTS drat aktivni pro prislusny kanal */
int 
ser_cts(byte kanal);

/* Vraci 1, pokud vysilani stale probiha, 0 pokud uz dobehlo. */
int 
ser_ostat(byte kanal);

/* Vraci 1, pokud prijimani stale probiha, 0 pokud uz dobehlo. */
int 
ser_istat(byte kanal);

/* Funkce pro praci s buffery serioveho portu *******************************/

#if (UART0_BUF > 0) || (UART1_BUF > 0)

/* Vyprazdni vstupni buffer */
void 
ser_flushbuf(byte kanal);

/* Vyprazdni vstupni buffer, a vyprazdnuje jej tak dlouho dokud v bajtech, ktere
 * prichazeji, neni alespon time jiffies pauza. 
 */
void 
ser_purgebuf(byte kanal, uint16 time);

/* Vraci pocet bajtu ve vstupnim bufferu seriaku */
int
ser_ibuflen(byte kanal);

/* vraci true pokud doslo k preteceni bufferu, false pokud ne */
bool 
ser_isoverflow(byte kanal);

#endif

#ifdef __CKDM__

#if (UART0_BUF > 0)
/* vstupni buffer */
extern volatile byte rxbuf0[UART0_BUF];
/* pocet prijatych bajtu ve vstupnim bufferu, UART0_BUF + 1 znamena 
 * preteceni */
extern volatile byte rxbuf0cnt;
#endif 

#if (UART1_BUF > 0)
/* vstupni buffer */
extern volatile byte rxbuf1[UART1_BUF];
/* pocet prijatych bajtu ve vstupnim bufferu, UART1_BUF + 1 znamena 
 * preteceni */
extern volatile byte rxbuf1cnt;
#endif 

#endif /* __CKDM__ */

#endif /* _UART_H_ */
